#include "burp.h"
#include "prog.h"

static long totalcounter=0;
static long filecounter=0;
static long changedcounter=0;
static long unchangedcounter=0;
static long newcounter=0;
static long directorycounter=0;
static long specialcounter=0;
static long hardlinkcounter=0;
static long softlinkcounter=0;
static long warningcounter=0;
static long bytecounter=0;
static long recvbytecounter=0;
static long sentbytecounter=0;

void reset_filecounter(void)
{
	totalcounter=0;
	filecounter=0;
	changedcounter=0;
	unchangedcounter=0;
	newcounter=0;
	directorycounter=0;
	specialcounter=0;
	hardlinkcounter=0;
	softlinkcounter=0;
	warningcounter=0;
	bytecounter=0;
	recvbytecounter=0;
	sentbytecounter=0;
}

static void pcounter(const char *str, long counter)
{
	if(counter) logp(str, counter);
}

static void end_filecounter(int print, int restore)
{
	if(print && !restore) printf(" %lu\n\n", totalcounter);
	else logp("\n");
	if(restore)
	{
		logp("Restored files:  %lu\n", filecounter);
	}
	else
	{
		pcounter("Files:           %lu\n", filecounter);
		pcounter("New files:       %lu\n", newcounter);
		pcounter("Changed files:   %lu\n", changedcounter);
		pcounter("Unchanged files: %lu\n", unchangedcounter);
	}
	pcounter("Directories:     %lu\n", directorycounter);
	pcounter("Soft links:      %lu\n", softlinkcounter);
	pcounter("Hard links:      %lu\n", hardlinkcounter);
	pcounter("Special files:   %lu\n", specialcounter);
	//logp("\n");
	pcounter("Total:           %lu\n", totalcounter);
	//logp("\n");
	pcounter("Warnings:        %lu\n", warningcounter);
	pcounter("Bytes received   %lu\n", recvbytecounter);
	pcounter("Bytes sent:      %lu\n", sentbytecounter);
	pcounter("Bytes in backup: %lu\n", bytecounter);
	//logp("\n");
	reset_filecounter();
}

void end_filecounter_backup(int print)
{
	end_filecounter(print, 0);
}

void end_filecounter_restore(int print)
{
	end_filecounter(print, 1);
}

void do_filecounter(char c, int print)
{
	if(print)
	{
		if(!totalcounter && !warningcounter) printf("\n");
		printf("%c", c);
	}
	switch(c)
	{
		case 'f': ++filecounter; break;
		case 'x': ++changedcounter; break;
		case 'F': ++newcounter; break;
		case 'd': ++directorycounter; break;
		case 's': ++specialcounter; break;
		case 'l': ++softlinkcounter; break;
		case 'L': ++hardlinkcounter; break;
		case 'w': ++warningcounter; return; // do not add to total
	}
	if(!(++totalcounter%64) && print) printf(" %lu\n", totalcounter);
	fflush(stdout);
}

void do_filecounter_bytes(unsigned long long bytes)
{
	bytecounter+=bytes;
}

void do_filecounter_sentbytes(unsigned long long bytes)
{
	sentbytecounter+=bytes;
}

void do_filecounter_recvbytes(unsigned long long bytes)
{
	recvbytecounter+=bytes;
}
